@@ -21,17 +21,21 @@ DEFAULT_EXTENSION_SETTINGS = {
         enabled: true
     },
     mprisMediaSessions: {
-        enabled: false
+        enabled: true
     },
     kdeconnect: {
         enabled: true
     },
     downloads: {
-        enabled: true
+        enabled: true,
+        saveOriginUrl: false
     },
     tabsrunner: {
         enabled: true
     },
+    purpose: {
+        enabled: true
+    },
     breezeScrollBars: {
         // this breaks pages in interesting ways, disable by default
         enabled: false
@@ -40,4 +44,5 @@ DEFAULT_EXTENSION_SETTINGS = {
 
 IS_FIREFOX = (typeof InstallTrigger !== "undefined"); // heh.
 
-SUPPORTED_PLATFORMS = ["linux", "openbsd"];
+// NOTE if you change this, make sure to adjust the error message shown in action_popup.html
+SUPPORTED_PLATFORMS = ["linux", "openbsd", "freebsd"];
