@@ -26,8 +26,9 @@
 #include <QCoreApplication>
 #include <QDBusConnection>
 #include <QDBusObjectPath>
-#include <QDebug>
+#include <QGuiApplication>
 #include <QImageReader>
+#include <QTimer>
 
 #include "mprisroot.h"
 #include "mprisplayer.h"
@@ -51,6 +52,10 @@ MPrisPlugin::MPrisPlugin(QObject *parent)
         return;
     }
 
+    m_propertyChangeSignalTimer.setInterval(0);
+    m_propertyChangeSignalTimer.setSingleShot(true);
+    connect(&m_propertyChangeSignalTimer, &QTimer::timeout, this, &MPrisPlugin::sendPropertyChanges);
+
     m_possibleLoopStatus = {
         {QStringLiteral("None"), false},
         {QStringLiteral("Track"), true},
@@ -71,25 +76,44 @@ void MPrisPlugin::emitPropertyChange(const QDBusAbstractAdaptor *interface, cons
     // TODO figure out encoding encoding
     const QString interfaceName = QString::fromUtf8(interface->metaObject()->classInfo(0).value());
 
-    QDBusMessage signal = QDBusMessage::createSignal(
-        QStringLiteral("/org/mpris/MediaPlayer2"),
-        QStringLiteral("org.freedesktop.DBus.Properties"),
-        QStringLiteral("PropertiesChanged")
-    );
+    const QMetaProperty prop = metaObject()->property(metaObject()->indexOfProperty(propertyName));
 
-    QMetaProperty prop = metaObject()->property(metaObject()->indexOfProperty(propertyName));
+    const QString propertyNameString = QString::fromUtf8(prop.name());
+    const QVariant value = prop.read(this);
 
-    signal.setArguments({
-        interfaceName,
-        QVariantMap{ // updated
-            {QString::fromUtf8(prop.name()), prop.read(this)}
-        },
-        QStringList() // invalidated
-    });
+    m_pendingPropertyChanges[interfaceName][propertyNameString] = value;
 
-    QDBusConnection::sessionBus().send(signal);
+    if (!m_propertyChangeSignalTimer.isActive()) {
+        m_propertyChangeSignalTimer.start();
+    }
+}
 
-    //qDebug() << "emitted change on iface" << interfaceName << "for prop" << propertyName << "which was" << signal;
+void MPrisPlugin::sendPropertyChanges()
+{
+    for (auto it = m_pendingPropertyChanges.constBegin(), end = m_pendingPropertyChanges.constEnd(); it != end; ++it) {
+        const QString &interfaceName = it.key();
+
+        const QVariantMap &changes = it.value();
+        if (changes.isEmpty()) {
+            continue;
+        }
+
+        QDBusMessage signal = QDBusMessage::createSignal(
+            QStringLiteral("/org/mpris/MediaPlayer2"),
+            QStringLiteral("org.freedesktop.DBus.Properties"),
+            QStringLiteral("PropertiesChanged")
+        );
+
+        signal.setArguments({
+            interfaceName,
+            changes,
+            QStringList() // invalidated
+        });
+
+        QDBusConnection::sessionBus().send(signal);
+    }
+
+    m_pendingPropertyChanges.clear();
 }
 
 bool MPrisPlugin::registerService()
@@ -129,6 +153,7 @@ void MPrisPlugin::handleData(const QString &event, const QJsonObject &data)
         m_canGoNext = false;
         m_canGoPrevious = false;
         m_pageTitle.clear();
+        m_tabTitle.clear();
         m_url.clear();
         m_mediaSrc.clear();
         m_title.clear();
@@ -140,7 +165,10 @@ void MPrisPlugin::handleData(const QString &event, const QJsonObject &data)
         m_position = 0;
     } else if (event == QLatin1String("playing")) {
         setPlaybackStatus(QStringLiteral("Playing"));
-        m_pageTitle = data.value(QStringLiteral("tabTitle")).toString();
+
+        m_pageTitle = data.value(QStringLiteral("pageTitle")).toString();
+        m_tabTitle = data.value(QStringLiteral("tabTitle")).toString();
+
         m_url = QUrl(data.value(QStringLiteral("url")).toString());
         m_mediaSrc = QUrl(data.value(QStringLiteral("mediaSrc")).toString());
 
@@ -175,6 +203,18 @@ void MPrisPlugin::handleData(const QString &event, const QJsonObject &data)
             setLoopStatus(loop ? QStringLiteral("Track") : QStringLiteral("None"));
         }
 
+        const bool fullscreen = data.value(QStringLiteral("fullscreen")).toBool();
+        if (m_fullscreen != fullscreen) {
+            m_fullscreen = fullscreen;
+            emitPropertyChange(m_root, "Fullscreen");
+        }
+
+        const bool canSetFullscreen = data.value(QStringLiteral("canSetFullscreen")).toBool();
+        if (m_canSetFullscreen != canSetFullscreen) {
+            m_canSetFullscreen = canSetFullscreen;
+            emitPropertyChange(m_root, "CanSetFullscreen");
+        }
+
         processMetadata(data.value(QStringLiteral("metadata")).toObject()); // also emits metadataChanged signal
         processCallbacks(data.value(QStringLiteral("callbacks")).toArray());
 
@@ -225,6 +265,12 @@ void MPrisPlugin::handleData(const QString &event, const QJsonObject &data)
         if (oldTitle != effectiveTitle()) {
             emitPropertyChange(m_player, "Metadata");
         }
+    } else if (event == QLatin1String("fullscreenchange")) {
+        const bool fullscreen = data.value(QStringLiteral("fullscreen")).toBool();
+        if (m_fullscreen != fullscreen) {
+            m_fullscreen = fullscreen;
+            emitPropertyChange(m_root, "Fullscreen");
+        }
     } else {
         qWarning() << "Don't know how to handle mpris event" << event;
     }
@@ -246,19 +292,7 @@ QString MPrisPlugin::identity() const
 
 QString MPrisPlugin::desktopEntry() const
 {
-    switch (Settings::self().environment()) {
-    case Settings::Environment::Unknown: return QString();
-    case Settings::Environment::Chrome: return QStringLiteral("google-chrome");
-    // TODO account for distros that want to be super special and install these with a different name
-    // perhaps we could do a KService lookup of some sort
-    case Settings::Environment::Chromium: return QStringLiteral("chromium-browser");
-    case Settings::Environment::Firefox: return QStringLiteral("firefox");
-    case Settings::Environment::Opera: return QStringLiteral("opera");
-    // This is what the official package on their website uses
-    case Settings::Environment::Vivaldi: return QStringLiteral("vivaldi-stable");
-    }
-
-    return QString();
+    return QGuiApplication::desktopFileName();
 }
 
 bool MPrisPlugin::canRaise() const
@@ -266,6 +300,23 @@ bool MPrisPlugin::canRaise() const
     return true; // really?
 }
 
+bool MPrisPlugin::fullscreen() const
+{
+    return m_fullscreen;
+}
+
+void MPrisPlugin::setFullscreen(bool fullscreen)
+{
+    sendData(QStringLiteral("setFullscreen"), {
+        {QStringLiteral("fullscreen"), fullscreen}
+    });
+}
+
+bool MPrisPlugin::canSetFullscreen() const
+{
+    return m_canSetFullscreen;
+}
+
 bool MPrisPlugin::canGoNext() const
 {
     return m_canGoNext;
@@ -376,7 +427,10 @@ QString MPrisPlugin::effectiveTitle() const
     if (!m_title.isEmpty()) {
         return m_title;
     }
-    return m_pageTitle;
+    if (!m_pageTitle.isEmpty()) {
+        return m_pageTitle;
+    }
+    return m_tabTitle;
 }
 
 QVariantMap MPrisPlugin::metadata() const
@@ -539,11 +593,17 @@ void MPrisPlugin::Quit()
 
 void MPrisPlugin::Next()
 {
+    if (!m_canGoNext) {
+        return;
+    }
     sendData(QStringLiteral("next"));
 }
 
 void MPrisPlugin::Previous()
 {
+    if (!m_canGoPrevious) {
+        return;
+    }
     sendData(QStringLiteral("previous"));
 }
 
